@@ -13,16 +13,54 @@ from dataclasses import dataclass, field
 from types import SimpleNamespace
 from pprint import pprint
 import numpy as np
+from prettytable import PrettyTable, PLAIN_COLUMNS
+
 
 from cdftpy.cdft1d.config import DATA_DIR
 from cdftpy.cdft1d.io_utils import read_array, read_geom, write_matrix_array
 from cdftpy.cdft1d.io_utils import read_key_value
-from cdftpy.cdft1d.io_utils import read_system
-from cdftpy.utils.rad_fft import RadFFT
+from cdftpy.cdft1d.io_utils import read_molecule
+from cdftpy.utils.rad_fft import RadFFT, fft_rgrid_iv
 from cdftpy.utils.units import hartree_2_kjmol, bohr_2_ang
+from itertools import combinations as comb
 
 logging.basicConfig(level=logging.INFO)
 
+@dataclass
+class Molecule1:
+
+    x: list[float] = field(default_factory=list)
+    y: list[float] = field(default_factory=list)
+    z: list[float] = field(default_factory=list)
+
+    sigma:  list[float] = field(default_factory=list)
+    eps:    list[float] = field(default_factory=list)
+    charge: list[float] = field(default_factory=list)
+
+    aname: list[str] = field(default_factory=list)
+    atype: list[str] = field(default_factory=list)
+
+    dipole: float = field(init=False)
+    nv: int = field(init=False)
+
+    def __post_init__(self):
+        nv = len(self.x)
+        if nv !=0:
+            self.nv = nv
+            q = self.charge
+            mu2 = np.dot(self.x, q) ** 2 + np.dot(self.y, q) ** 2 + np.dot(self.z, q) ** 2
+            self.dipole = np.sqrt(mu2)
+
+    @classmethod
+    def from_file(cls, filename):
+        solvent = read_molecule(filename)
+        solvent["nv"] = len(solvent["aname"])
+
+        return cls(**solvent)
+
+    def sm_k(self, kgrid):
+        return compute_sm_rigid_bond(self.x, self.y, self.z, kgrid)
+
 @dataclass
 class Molecule:
 
@@ -55,14 +93,9 @@ class Molecule:
     def sm_k(self, kgrid):
         return compute_sm_rigid_bond(self.x, self.y, self.z, kgrid)
 
-    def comp_dipole_moment(solvent_model):
-        x = solvent_model.x
-        y = solvent_model.y
-        z = solvent_model.z
-        q = solvent_model.charge
-
 @dataclass
 class Solvent(Molecule):
+    model: str
     filename: str
     file_location: str
 
@@ -78,11 +111,12 @@ class Solvent(Molecule):
 
     @classmethod
     def from_file(cls, filename, rism_patch=False):
-        solvent = read_system(filename,rism_patch=rism_patch)
+        solvent = read_molecule(filename, rism_patch=rism_patch)
         nv = len(solvent["aname"])
         solvent["nv"] = nv
         solvent["filename"] = os.path.abspath(filename)
         solvent["file_location"] = os.path.split(solvent["filename"][0])
+        solvent["model"] = os.path.basename(filename).split(".")[0]
 
         state = read_key_value(filename, "state")
 
@@ -98,6 +132,8 @@ class Solvent(Molecule):
 
     def zeta(self, beta):
         mu2 = self.dipole**2
+        if mu2 == 0:
+            return 0
         rho = self.density
         y3 = (hartree_2_kjmol / bohr_2_ang) * beta * 4 * np.pi * mu2 * rho / 3
         y3m = 1.0 - 1.0 / self.dielectric
@@ -137,6 +173,64 @@ class Solvent(Molecule):
                         fp.write(F"{s_k[i,j,k]}  ")
                 fp.write("\n")
 
+    def extend(self, rmax):
+
+        ifft = self.ifft
+        rgrid = ifft.rgrid
+
+        dr = rgrid[1] - rgrid[0]
+        nr_old = len(rgrid)
+        nr_new = int(rmax / dr)
+
+        if nr_new > nr_old:
+            s_r = np.apply_along_axis(ifft.to_rspace, 2, self.s_k)
+            dims = list(s_r.shape)
+            dims[-1] = nr_new
+            s_r_new = np.zeros(dims)
+            s_r_new[..., :nr_old] = s_r[..., :nr_old]
+            rgrid_new = fft_rgrid_iv(dr, nr_new)
+            self.ifft = RadFFT.from_rgrid(rgrid_new)
+            self.s_k = np.apply_along_axis(self.ifft.to_kspace, 2, s_r_new)
+
+    def distance_matrix(self):
+
+        xv = self.x
+        yv = self.y
+        zv = self.z
+
+        d = (
+                np.subtract.outer(xv, xv) ** 2
+                + np.subtract.outer(yv, yv) ** 2
+                + np.subtract.outer(zv, zv) ** 2
+        )
+        d = np.sqrt(d)
+
+        return d
+
+    def report(self):
+
+        nv = self.nv
+        print(f"Model: {self.model}")
+        tbl = PrettyTable()
+        tbl.set_style(PLAIN_COLUMNS)
+        tbl.field_names = ["site", "sigma(Å)", "epsilon(kj/mol)", "charge"]
+        for i in range(nv):
+            tbl.add_row([self.aname[i], self.sigma[i], self.eps[i], self.charge[i]])
+        tbl.align = "l"
+        tbl.align["charge"]="r"
+        print(tbl)
+        # tbl.left_padding_width = 2
+        # print(tbl)
+        print("Bonds")
+        aname = self.aname
+        d = self.distance_matrix()
+        for i, j in comb(range(nv), 2):
+            print(F"{aname[i]}-{aname[j]} {d[i, j]} Å"),
+        print(F"Density {self.density}")
+        print(F"Temp {self.temp}")
+        print(F"Dielectric {self.dielectric}")
+
+
 def solvent_model_locate(solvent_name):
     solvent = solvent_name + ".smdl"
     cwd = pathlib.Path.cwd()
@@ -182,16 +276,39 @@ def compute_sm_rigid_bond(xv, yv, zv, kgrid):
     return d2
 
 
-if __name__ == "__main__":
-    geom = read_geom("../data/s2.smdl")
-    params = dict(density=1.0)
-    # solvent = Solvent(**params,**geom)
-    solvent = Solvent.from_file("../data/s2.smdl", rism_patch=True)
 
-    structure_factor = dict (
-        tag="s_k",
-        kgrid = solvent.ifft.kgrid,
-        s_k = solvent.s_k
-    )
-    solvent.to_smdl_file("test1.dat", structure_factor)
-    write_matrix_array("sk.dat", solvent.ifft.kgrid, solvent.s_k, header="S(k)")
+if __name__ == "__main__":
+
+    solute = dict(name="Cl", charge=-1.0, sigma=4.83, eps=0.05349244)
+    m = Molecule1(**solute)
+    pass
+    # s = solvent.report()
+    #
+    # s.seek(0)
+    # for line in s:
+    #     print(line)
+
+    # nv = solvent.nv
+    # tbl = PrettyTable()
+    # tbl.set_style(PLAIN_COLUMNS)
+    # tbl.field_names = ["name", "type", "sigma", "epsilon", "charge"]
+    # for i in range(solvent.nv):
+    #     tbl.add_row([solvent.aname[i],solvent.atype[i],solvent.sigma[i],solvent.eps[i],solvent.charge[i]])
+    # print("Parameters")
+    # tbl.padding_width = 0
+    # print(tbl)
+    # # tbl.left_padding_width = 2
+    # # print(tbl)
+    # print("Bonds")
+    # aname = solvent.aname
+    # d = solvent.distance_matrix()
+    # for i,j in comb(range(nv), 2):
+    #     print(F{aname[i]}-{aname[j]}  {d[i,j]}")
+
+    # structure_factor = dict (
+    #     tag="s_k",
+    #     kgrid = solvent.ifft.kgrid,
+    #     s_k = solvent.s_k
+    # )
+    # solvent.to_smdl_file("test1.dat", structure_factor)
+    # write_matrix_array("sk.dat", solvent.ifft.kgrid, solvent.s_k, header="S(k)")
