@@ -32,6 +32,20 @@ def simple_2_plot(r, f1, f2, xrange=None, yrange=None, label=("f1","f2")):
     plt.grid()
     plt.show()
 
+def simple_22_plot(r1, f1,r2, f2, xrange=None, yrange=None, label=("f1","f2")):
+    # plt.plot(r,f1,marker='o',markevery=50,linestyle='solid')
+    plt.plot(r1,f1,linestyle='solid', label=label[0])
+    plt.plot(r2,f2,linestyle='dashed',label=label[1])
+    plt.legend()
+    if xrange is not None:
+        plt.xlim(xrange)
+    if yrange is not None:
+        plt.ylim(yrange)
+
+    # plt.axhline(0)
+    plt.grid()
+    plt.show()
+
 def simple_3_plot(r, f1, f2, f3, xrange=None, yrange=None):
     plt.plot(r,f1,marker='o',markevery=50,linestyle='solid')
     plt.plot(r,f2,marker='x',markevery=50,linestyle='solid')
